@@ -84,6 +84,9 @@ struct ArgumentProcessingState
   // the compiler, not the preprocessor, and that also should not be part of the
   // hash identifying the result.
   Args compiler_only_args_no_hash;
+
+  // Whether to include the full command line in the hash.
+  bool hash_full_command_line = false;
 };
 
 bool
@@ -687,19 +690,27 @@ process_arg(Context& ctx,
     return nullopt;
   }
 
-  if (config.compiler_type() != CompilerType::clang
+  if (config.compiler_type() == CompilerType::gcc
       && (args[i] == "-fcolor-diagnostics"
           || args[i] == "-fno-color-diagnostics")) {
-    // Special case: If a non-Clang compiler gets -f(no-)color-diagnostics we'll
-    // bail out and just execute the compiler. The reason is that we don't
-    // include -f(no-)color-diagnostics in the hash so there can be a false
-    // cache hit in the following scenario:
+    // Special case: If a GCC compiler gets -f(no-)color-diagnostics we'll bail
+    // out and just execute the compiler. The reason is that we don't include
+    // -f(no-)color-diagnostics in the hash so there can be a false cache hit in
+    // the following scenario:
     //
     //   1. ccache gcc -c example.c                      # adds a cache entry
     //   2. ccache gcc -c example.c -fcolor-diagnostics  # unexpectedly succeeds
     return Statistic::unsupported_compiler_option;
   }
 
+  // In the "-Xclang -fcolor-diagnostics" form, -Xclang is skipped and the
+  // -fcolor-diagnostics argument which is passed to cc1 is handled below.
+  if (args[i] == "-Xclang" && i < args.size() - 1
+      && args[i + 1] == "-fcolor-diagnostics") {
+    state.compiler_only_args_no_hash.push_back(args[i]);
+    ++i;
+  }
+
   if (args[i] == "-fcolor-diagnostics" || args[i] == "-fdiagnostics-color"
       || args[i] == "-fdiagnostics-color=always") {
     state.color_diagnostics = ColorDiagnostics::always;
@@ -754,6 +765,10 @@ process_arg(Context& ctx,
     return nullopt;
   }
 
+  if (args[i] == "-frecord-gcc-switches") {
+    state.hash_full_command_line = true;
+  }
+
   // Options taking an argument that we may want to rewrite to relative paths to
   // get better hit rate. A secondary effect is that paths in the standard error
   // output produced by the compiler will be normalized.
@@ -955,18 +970,51 @@ process_args(Context& ctx)
 
   state.common_args.push_back(args[0]); // Compiler
 
+  optional<Statistic> argument_error;
   for (size_t i = 1; i < args.size(); i++) {
-    auto error = process_arg(ctx, args, i, state);
-    if (error) {
-      return *error;
+    const auto error = process_arg(ctx, args, i, state);
+    if (error && !argument_error) {
+      argument_error = error;
     }
   }
 
+  // Don't try to second guess the compiler's heuristics for stdout handling.
+  if (args_info.output_obj == "-") {
+    LOG_RAW("Output file is -");
+    return Statistic::output_to_stdout;
+  }
+
+  // Determine output object file.
+  const bool implicit_output_obj = args_info.output_obj.empty();
+  if (implicit_output_obj && !args_info.input_file.empty()) {
+    string_view extension = state.found_S_opt ? ".s" : ".o";
+    args_info.output_obj =
+      Util::change_extension(Util::base_name(args_info.input_file), extension);
+  }
+
+  // On argument processing error, return now since we have determined
+  // args_info.output_obj which is needed to determine the log filename in
+  // CCACHE_DEBUG mode.
+  if (argument_error) {
+    return *argument_error;
+  }
+
   if (state.generating_debuginfo_level_3 && !config.run_second_cpp()) {
+    // Debug level 3 makes line number information incorrect when compiling
+    // preprocessed code.
     LOG_RAW("Generating debug info level 3; not compiling preprocessed code");
     config.set_run_second_cpp(true);
   }
 
+#ifdef __APPLE__
+  // Newer Clang versions on macOS are known to produce different debug
+  // information when compiling preprocessed code.
+  if (args_info.generating_debuginfo && !config.run_second_cpp()) {
+    LOG_RAW("Generating debug info; not compiling preprocessed code");
+    config.set_run_second_cpp(true);
+  }
+#endif
+
   handle_dependency_environment_variables(ctx, state);
 
   if (args_info.input_file.empty()) {
@@ -1007,6 +1055,10 @@ process_args(Context& ctx)
     args_info.actual_language.find("-header") != std::string::npos
     || Util::is_precompiled_header(args_info.output_obj);
 
+  if (args_info.output_is_precompiled_header && implicit_output_obj) {
+    args_info.output_obj = args_info.input_file + ".gch";
+  }
+
   if (args_info.output_is_precompiled_header
       && !(config.sloppiness() & SLOPPY_PCH_DEFINES)) {
     LOG_RAW(
@@ -1054,22 +1106,6 @@ process_args(Context& ctx)
     config.set_cpp_extension(extension_for_language(p_language).substr(1));
   }
 
-  // Don't try to second guess the compilers heuristics for stdout handling.
-  if (args_info.output_obj == "-") {
-    LOG_RAW("Output file is -");
-    return Statistic::output_to_stdout;
-  }
-
-  if (args_info.output_obj.empty()) {
-    if (args_info.output_is_precompiled_header) {
-      args_info.output_obj = args_info.input_file + ".gch";
-    } else {
-      string_view extension = state.found_S_opt ? ".s" : ".o";
-      args_info.output_obj = Util::change_extension(
-        Util::base_name(args_info.input_file), extension);
-    }
-  }
-
   if (args_info.seen_split_dwarf) {
     args_info.output_dwo = Util::change_extension(args_info.output_obj, ".dwo");
   }
@@ -1220,6 +1256,9 @@ process_args(Context& ctx)
   if (config.run_second_cpp()) {
     extra_args_to_hash.push_back(state.dep_args);
   }
+  if (state.hash_full_command_line) {
+    extra_args_to_hash.push_back(ctx.orig_args);
+  }
 
   if (diagnostics_color_arg) {
     compiler_args.push_back(*diagnostics_color_arg);
